/*
*	Marcin Wyrzykowski H1ISI				Praca Domowa ZPR
*	Algorytm Includes
*
* Algorytm Includes sprawdza czy posortowany zakres elementw zawiera wszytkie
* elementy znajdujce si w innym posortowanym zakresie elementw.
* Relacja mniejszoci elementw moe zosta okrelony przez binarny predykat.
*
*/

/*
Prototypy:
includes jest przecion nazw, istniej dwie wersje funkcje:

template <class InputIterator1, class InputIterator2>
bool includes(InputIterator1 first1, InputIterator1 last1,
              InputIterator2 first2, InputIterator2 last2);

template <class InputIterator1, class InputIterator2, class StrictWeakOrdering>
bool includes(InputIterator1 first1, InputIterator1 last1,
              InputIterator2 first2, InputIterator2 last2, 
              StrictWeakOrdering comp);

Pierwsza wersja porwnuje elementy wykorzystujc "operator<".
Druga wersja porwnuje elementy wykorzystujc podany przez uytkownika obiekt funkcyjny.

Opis parametrw:
	first1 - iterator wskazujcy pierwszy element pierwszego zakresu elementw
	last1 - iterator wskazujcy element za ostatnim elemetntem pierwszego zakresu elementw
	first2 - iterator wskazujcy pierwszy element drugiego zakresu elementw
	last2 - iterator wskazujcy element za ostatnim elemetntem drugiego zakresu elementw
	comp - obiekt funkcyjny okrelajcy relacje mniejszoi. Pobiera dwa argumenty i	
		zwraca "true" jeeli warunek jest spenimu lub "false" wpp.

Warunki wstpne dla argumentw:
	Zarwno pierwszy i drugi zakres elementw powinien by posortowany w porzdku rosncym 
	z uyciem odpowiedniego operatora:
		- dla pierwszej wersji - "operator<",
		- dla drugiej wersji - podany przez uytkownika obiekt funkcyjny "comp".
	InputIterator1 i InputIterator2 s iteratorami do tych samych typw.

Zwracana warto:
	"true" jeli dla kadego elementu z [first2, last2) istnieje odpowiednik w [first1, last1),
	wpp. "false". 
	Jeli elementy powtarzaj si istotna jest liczba powtrze.
	
Zoono:
	Liniowa. Zero porwna jeli jeden z zakresw jest pusty,
	wpp. maksymalnie 2 * ((last1 - first1) + (last2 - first2)) - 1 porwna. 

*/

#include <algorithm>
#include <iostream>
#include <string>
#include <vector>
#include <deque>
#include <cmath>
#include <functional>

using namespace std;


int main()
{
///////////////////////////////////////////////////////////////////////////////
//Przyklad1 dla pierwszej wersji funkcji incudes()
	cout<<"Przyklad1 - pierwsza wersja funkcji incudes()"<<endl;

	bool result;

	string s1("abcde");
	string s2("abd");
	string s3("ax");
	string s4("aaabccde");
	string s5("aaacc");
	string s6("aaaccc");
	
	//tworzymy wektory char'w,
	//wektory powinny zawiera posortowane leksykograficznie elementy
	vector<char> vector1(s1.begin(), s1.end());
	vector<char> vector2(s2.begin(), s2.end());
	vector<char> vector3(s3.begin(), s3.end());
	vector<char> vector4(s4.begin(), s4.end());
	vector<char> vector5(s5.begin(), s5.end());
	vector<char> vector6(s6.begin(), s6.end());
	
	//includes sprawdza czy dla wszystkich charw zawartych w vectorze drugim istniej
	//odpowiedniki w wektorze pierwszym. Istotna jest liczba powtrze elementw.
	//wydruki postaci 'xx' zawarte w 'yy', gdzie 'xx' oznacza kolejne chary 
	cout <<"'"<< s2 << "' zawarte w '" << s1 << "'- ";
	result = includes(vector1.begin(), vector1.end(), vector2.begin(), vector2.end());
	if (result) cout<<"true"; else cout<<"false";
	cout<< endl;

	cout <<"'"<< s3 << "' zawarte w '" << s1 << "'- ";
	result = includes(vector1.begin(), vector1.end(), vector3.begin(), vector3.end());
	if (result) cout<<"true"; else cout<<"false";
	cout<< endl;

	cout <<"'"<< s5 << "' zawarte w '" << s4 << "'- ";
	result = includes(vector4.begin(), vector4.end(), vector5.begin(), vector5.end());
	if (result) cout<<"true"; else cout<<"false";
	cout<< endl;

	cout <<"'"<< s6 << "' zawarte w '" << s4 << "'- ";
	result = includes(vector4.begin(), vector4.end(), vector6.begin(), vector6.end());
	if (result) cout<<"true"; else cout<<"false";
	cout<< endl;

////////////////////////////////////////////////////////////////////////////////////
//Przyklad dla drugiej wersji includes 
// - z uzyciem obiektu funkcyjnego do porownywania.

	cout<<endl<<"Przyklad2 - z uzyciem obiektu funkcyjnego do porownywania"<<endl;
	
	//obiekt funkcyjny porwnujcy wartoci bezwgldne elementw
	struct less_mag_fabs : public binary_function<float, float, bool> {
		bool operator()(float x, float y) { return fabs(x) < fabs(y); }
    };
	
	deque<float> deque_;
	deque_.push_back(-4);
	deque_.push_back(-6);
	deque_.push_back(3);
	deque_.push_back(5);

	vector<float> vector_;
	vector_.push_back(-4);
	vector_.push_back(3);
	vector_.push_back(6);

	//sortowanie z uwzgldnieniem wartociach bezwzgldnych elementw
	sort(deque_.begin(), deque_.end(), less_mag_fabs());
	sort(vector_.begin(), vector_.end(), less_mag_fabs());
	
	//includes sprawdza czy dla wartoci bezwgldnej kolejnych elementw z vector_
	//istniej w deque_ odpowiadajce elementy o rwnej wartoci bezwgldnej.
	cout <<"vector_ zawarte w deque_ - ";
	result = includes(deque_.begin(), deque_.end(), vector_.begin(), vector_.end(), less_mag_fabs());
	if (result) cout<<"true"; else cout<<"false";
	cout<<endl;

  return 0;
}
